<?php

namespace Drupal\entity_template_ui\Controller;

use Drupal\Core\Ajax\AjaxHelperTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Url;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\TemplateComponentManager;
use Drupal\entity_template\TemplateManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ChooseConditionController extends ControllerBase {
  use AjaxHelperTrait;

  /**
   * @var \Drupal\entity_template\TemplateManager
   */
  protected $templateManager;

  /**
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * @var \Drupal\Core\Plugin\Context\ContextHandlerInterface
   */
  protected $contextHandler;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.entity_template.template'),
      $container->get('plugin.manager.condition'),
      $container->get('context.handler')
    );
  }

  /**
   * ChooseTemplateController constructor.
   *
   * @param \Drupal\entity_template\TemplateManager $template_manager
   */
  public function __construct(TemplateManager $template_manager, ConditionManager $condition_manager, ContextHandlerInterface $context_handler) {
    $this->templateManager = $template_manager;
    $this->conditionManager = $condition_manager;
    $this->contextHandler = $context_handler;
  }

  /**
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   */
  public function build(BlueprintStorageInterface $blueprint_storage, $template_key) {
    $template = $blueprint_storage->getTemplate($template_key);
    $available_definitions = $this->conditionManager->getDefinitions();
    $available_definitions = $this->contextHandler->filterPluginDefinitionsByContexts(
      $template->getContexts(TRUE),
      $available_definitions
    );

    $build['condition_categories'] = [
      '#type' => 'container',
    ];

    foreach ($available_definitions as $id => $available_definition) {
      $category = isset($available_definition['category']) ? $available_definition['category'] : 'Other';

      if (!isset($build[$category])) {
        $build[$category]['#type'] = 'details';
        $build[$category]['#open'] = TRUE;
        $build[$category]['#title'] = $category;
        $build[$category]['links'] = [
          '#theme' => 'links',
          '#links' => [],
        ];
      }

      $attributes = $this->getAjaxAttributes();
      $build[$category]['links']['#links'][] = [
        'title' => $available_definition['label'],
        'url' => Url::fromRoute('entity_template_ui.edit_template.add_condition', [
            'template_blueprint_provider' => $blueprint_storage->getProvider()->getPluginId(),
            'blueprint_storage' => $blueprint_storage->getProviderKey(),
            'template_key' => $template_key,
            'plugin_id' => $id,
          ]),
        'attributes' => $attributes,
      ];
    }

    return $build;
  }

  /**
   * Get dialog attributes if an ajax request.
   *
   * @return array
   *   The attributes array.
   */
  protected function getAjaxAttributes() {
    if ($this->isAjax()) {
      return [
        'class' => ['use-ajax'],
        'data-dialog-type' => 'dialog',
        'data-dialog-renderer' => 'off_canvas',
      ];
    }
    return [];
  }
}
