<?php

namespace Drupal\entity_template_ui\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormFactoryInterface;
use Drupal\Core\Plugin\PluginWithFormsInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\BlueprintTempstoreRepository;
use Drupal\entity_template\TemplateManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ConfigureTemplateForm extends FormBase {

  /**
   * @var \Drupal\entity_template\TemplateManager
   */
  protected $templateManager;

  /**
   * @var \Drupal\entity_template\BlueprintStorageInterface
   */
  protected $blueprintStorage;

  /**
   * @var \Drupal\Core\Plugin\PluginFormFactoryInterface
   */
  protected $pluginFormFactory;

  /**
   * @var \Drupal\entity_template\BlueprintTempstoreRepository
   */
  protected $blueprintTempstore;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $plugin;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.entity_template.template'),
      $container->get('plugin_form.factory'),
      $container->get('entity_template.blueprint_tempstore_repository')
    );
  }

  public function __construct(
    TemplateManager $template_manager,
    PluginFormFactoryInterface $plugin_form_factory,
    BlueprintTempstoreRepository $blueprint_tempstore_repository
  ) {
    $this->templateManager = $template_manager;
    $this->pluginFormFactory = $plugin_form_factory;
    $this->blueprintTempstore = $blueprint_tempstore_repository;
  }

  /**
   * Returns a unique string identifying the form.
   *
   * The returned ID should be a unique string that can be a valid PHP function
   * name, since it's used in hook implementation names such as
   * hook_form_FORM_ID_alter().
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'configure_template_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    BlueprintStorageInterface $blueprint_storage = NULL,
    $template_id = ''
  ) {
    $this->blueprintStorage = $blueprint_storage;
    if (!$this->plugin) {
      $this->plugin = $this->blueprintStorage->getTemplate($template_id);
    }

    if (
      ($this->plugin instanceof PluginWithFormsInterface) &&
      $this->plugin->hasFormClass("configure")
    ) {
      $plugin_form = $this->pluginFormFactory->createInstance(
        $this->plugin,
        "configure"
      );

      $form = $plugin_form->buildConfigurationForm($form, $form_state);
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save Template'),
    ];

    return $form;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if (
      ($this->plugin instanceof PluginWithFormsInterface) &&
      $this->plugin->hasFormClass("configure")
    ) {
      $plugin_form = $this->pluginFormFactory->createInstance(
        $this->plugin,
        "configure"
      );

      $plugin_form->submitConfigurationForm($form, $form_state);
    }

    $this->messenger()->addWarning(new TranslatableMarkup('This blueprint has unsaved template changes.'));
    $this->blueprintTempstore->set($this->blueprintStorage);
    $form_state->setRedirectUrl(
      $this->blueprintStorage->getEditUrl()
    );
  }
}
