<?php

namespace Drupal\entity_template_ui\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\BlueprintTempstoreRepository;
use Symfony\Component\DependencyInjection\ContainerInterface;

class RemoveTemplateForm extends ConfirmFormBase {

  /**
   * @var \Drupal\entity_template\BlueprintStorageInterface
   */
  protected $blueprintStorage;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $template;

  /**
   * @var \Drupal\entity_template\BlueprintTempstoreRepository
   */
  protected $blueprintTempstoreRepository;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_template.blueprint_tempstore_repository')
    );
  }

  /**
   * RemoveTemplateForm constructor.
   *
   * @param \Drupal\entity_template\BlueprintTempstoreRepository $blueprint_tempstore_tepository
   */
  public function __construct(BlueprintTempstoreRepository $blueprint_tempstore_tepository)
  {
    $this->blueprintTempstoreRepository = $blueprint_tempstore_tepository;
  }

  /**
   * Returns the question to ask the user.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The form question. The page title will be set to this value.
   */
  public function getQuestion() {
    return new TranslatableMarkup(
      'Are you sure you want to remove @template from @blueprint?',
      [
        '@template' => $this->template->label(),
        '@blueprint' => $this->blueprintStorage->getTempstoreKey()
      ]
    );
  }

  /**
   * Returns the route to go to if the user cancels the action.
   *
   * @return \Drupal\Core\Url
   *   A URL object.
   */
  public function getCancelUrl() {
    return Url::fromRoute(
      'entity_template_ui.edit_template',
      [
        'template_blueprint_provider' => $this->blueprintStorage->getProvider()->getPluginId(),
        'blueprint_storage' => $this->blueprintStorage->getTempstoreKey(),
        'template_id' => $this->template->getKey(),
      ]
    );
  }

  /**
   * Returns a unique string identifying the form.
   *
   * The returned ID should be a unique string that can be a valid PHP function
   * name, since it's used in hook implementation names such as
   * hook_form_FORM_ID_alter().
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'remove_template_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state, BlueprintStorageInterface $blueprint_storage = NULL, $template_id = NULL) {
    $this->blueprintStorage = $blueprint_storage;
    $this->template = $this->blueprintStorage->getTemplate($template_id);

    return parent::buildForm($form, $form_state);
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->blueprintStorage->removeTemplate($this->template->getKey());
    $this->blueprintTempstoreRepository->set($this->blueprintStorage);

    $form_state->setRedirectUrl($this->blueprintStorage->getEditUrl());
  }
}
