<?php

namespace Drupal\entity_template_ui\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Component\TemplateContextAwareComponentInterface;

class SwapComponentForm extends ComponentFormBase {

  /**
   * @var string
   */
  protected $componentId;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'swap_component_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    $op = "swap",
    BlueprintStorageInterface $blueprint_storage = NULL,
    $template_key = '',
    $sub_template = '',
    $component_id = '',
    $other_plugin = ''
  ) {
    $this->componentId = $component_id;
    $this->initializeTemplate($blueprint_storage, $template_key, $sub_template);

    if (!$this->plugin) {
      /** @var \Drupal\entity_template\Plugin\EntityTemplate\Component\SwappableComponentInterface $original_plugin */
      $original_plugin = $this->template->getComponent($component_id);
      $new_plugin = $this->componentManager->createInstance(
        $other_plugin,
        [
          'uuid' => $component_id,
        ]
      );
      if ($new_plugin instanceof TemplateContextAwareComponentInterface) {
        $new_plugin->setTemplate($this->template);
      }
      $this->plugin = $original_plugin->swapConfigurationTo($new_plugin);
    }

    return parent::doBuildForm($form, $form_state,'swap');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state, string $op = "swap") {
    parent::submitForm($form, $form_state, "swap");

    $this->template->getComponents()->set(
      $this->componentId,
      $this->plugin
    );
    $this->blueprintTempstore->set($this->blueprintStorage);

    $form_state->setRedirectUrl(
      $this->blueprintStorage->getEditTemplateUrl($this->rootTemplate->getKey())
    );
  }

}
