<?php

namespace Drupal\entity_template;

interface BlueprintInterface extends ContextAwareInterface {

  /**
   * Get the blueprint label.
   *
   * @return string
   *   The label.
   */
  public function label() : string;

  /**
   * @param \Drupal\Core\Plugin\Context\ContextInterface[] $parameters
   *
   * @return \Drupal\entity_template\BlueprintResult
   */
  public function execute(array $parameters = []);

  /**
   * The priority of this blueprint.
   *
   * @return integer
   */
  public function getPriority();

  /**
   * Set the priority of this blueprint
   *
   * @param integer $priority
   *
   * @return static
   */
  public function setPriority($priority);

  /**
   * Get the builder this blueprint is for.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface
   */
  public function getBuilder();

  /**
   * Get the templates that form part of this blueprint.
   *
   * @return \Drupal\Core\Plugin\DefaultLazyPluginCollection
   */
  public function getTemplates();

  /**
   * Get the template plugin.
   *
   * @param $id
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  public function getTemplate($id);

  /**
   * Get the configuration for the templates.
   *
   * @return array
   */
  public function getTemplatesConfiguration();
}
