<?php

namespace Drupal\entity_template;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Plugin\DefaultLazyPluginCollection;

/**
 * Class BlueprintLazyTemplatesCollection
 *
 * @package Drupal\entity_template
 */
class BlueprintLazyTemplatesCollection extends DefaultLazyPluginCollection {

  /**
   * @var \Drupal\entity_template\BlueprintInterface
   */
  protected $blueprint;

  /**
   * BlueprintLazyTemplatesCollection constructor.
   *
   * @param \Drupal\Component\Plugin\PluginManagerInterface $manager
   * @param \Drupal\entity_template\BlueprintInterface $blueprint
   * @param array $configurations
   */
  public function __construct(PluginManagerInterface $manager, BlueprintInterface $blueprint, array $configurations = []) {
    parent::__construct($manager, $configurations);

    $this->blueprint = $blueprint;
  }

  /**
   * {@inheritdoc}
   */
  protected function initializePlugin($instance_id) {
    $configuration = isset($this->configurations[$instance_id]) ? $this->configurations[$instance_id] : [];
    if (!isset($configuration[$this->pluginKey])) {
      throw new PluginNotFoundException($instance_id);
    }
    $this->set($instance_id, $this->manager->createInstance(
      $configuration[$this->pluginKey],
      $configuration + ['blueprint' => $this->blueprint]
    ));
  }

}
