<?php


namespace Drupal\entity_template\Plugin\EntityTemplate\Component;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\TypedData\TypedDataInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItem;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;

/**
 * Data select component to massage date values.
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\Component
 */
class DateTimeFieldDataSelectComponent extends FieldDataSelectComponent {

  /**
   * {@inheritdoc}
   */
  protected function getMassagedValue(TypedDataInterface $data) {
    if ($data->getDataDefinition()->getDataType() === 'datetime_iso8601') {
      return \DateTime::createFromFormat(\DateTime::ATOM, $data->getValue())
        ->setTimezone(new \DateTimeZone('UTC'))
        ->format(
          $this->getFieldDefinition()->getSetting('datetime_type') == DateTimeItem::DATETIME_TYPE_DATETIME ?
            DateTimeItemInterface::DATETIME_STORAGE_FORMAT :
            DateTimeItemInterface::DATE_STORAGE_FORMAT
        );
    }

    if ($data->getDataDefinition()->getDataType() === 'timestamp') {
      return DrupalDateTime::createFromTimestamp($data->getValue())
        ->format(
          $this->getFieldDefinition()->getSetting('datetime_type') == DateTimeItem::DATETIME_TYPE_DATETIME ?
            DateTimeItemInterface::DATETIME_STORAGE_FORMAT :
            DateTimeItemInterface::DATE_STORAGE_FORMAT
        );
    }

    return parent::getMassagedValue($data);
  }

}
